@@ -3,7 +3,6 @@
 package JMX::Jmx4Perl::J4psh::Command::MBean;
 use strict;
 use base qw(JMX::Jmx4Perl::J4psh::Command);
-use JMX::Jmx4Perl::Util;
 use JMX::Jmx4Perl::Request;
 use Data::Dumper;
 use JSON;
@@ -42,15 +41,29 @@ sub domain_commands {
             "cd" => { 
                      desc => "Enter a domain",
                      proc => sub { 
-                         my $domain = join '',@_;
-                         $self->_cd_absolute($domain);
+                         my $domain = shift;
+                         my $prop;
+                         if ($domain) {
+                             $domain =~ s/:+$//;
+                             ($domain,$prop) = split(/:/,$domain,2) if $domain =~ /:/;
+                         }    
+                         $self->_cd_domain($domain);
+                         if ($prop) {
+                             eval {
+                                 $self->_cd_mbean($domain,$prop);
+                             };
+                             if ($@) {
+                                 # We already entered the domain successfully
+                                 $self->pop_off_stack;
+                                 die $@;
+                             }
+                         } 
                      },
                      args => $self->complete->mbeans(all => 1),
-                    },
+                    }
            };
 }
 
-
 sub property_commands { 
     my $self = shift;
     my $domain = shift;
@@ -64,26 +77,13 @@ sub property_commands {
             "cd" => { 
                      desc => "Enter a MBean",
                      proc => sub {
-                         #print Dumper([@_]);
-                         #print Devel::StackTrace->new->as_string;
-                         my $input = join '',@_;
+                         my $input = shift;
                          if (!$self->_handle_navigation($input)) {
-                             if ($input =~ /:/) {
-                                 $self->_cd_absolute($input);
-                             } else {
-                                 $self->_cd_mbean($domain,$input);
-                             }
+                             $self->_cd_mbean($domain,$input);
                          }
                      },
-                     args => $self->complete->mbeans(domain => $domain)
-                    },
-            "pwd" => {
-                      desc => "Print currently selected domain",
-                      proc => sub {
-                          my ($s,$r) = $self->color("domain_name","reset");
-                          print $s . $domain . $r,":\n";
-                      }
-                     }
+                     args => $self->complete->mbeans(domain => $domain),
+                    }
            };
 }
 
@@ -92,32 +92,15 @@ sub mbean_commands {
     my $mbean_props = shift;
     return {
             "ls" => { 
-                     desc => "List MBeans for a domain.",
-                     doc => <<EOT,
-List all MBeans within a domain. 
-
-The following options are supported:
-
-   -a: Attributes only
-   -o: Operations only
-
-Wildcards are supported for filtering
-EOT
+                     desc => "List MBeans for a domain",
                      proc => $self->cmd_show_mbean($mbean_props),
                      #args => $self->complete->mbean_attribs($mbean_props),
                     },
             "cd" => {
-                     desc => "Navigate up (..), to the top (/) or directly to another MBean",
+                     desc => "Navigate up (..) or to the top (/)",
                      proc => sub {
-                         my $input = join '',@_;
-                         if (!$self->_handle_navigation($input)) {
-                             if ($input =~ /:/) {
-                                 # "Absolute path"
-                                 $self->_cd_absolute($input);
-                             } else {
-                                 die "No MBean '",$input,"' known\n";
-                             }
-                         };
+                         my $input = shift;
+                         $self->_handle_navigation($input);
                      },
                     },
             "cat" => { 
@@ -136,13 +119,7 @@ EOT
                        proc => $self->cmd_execute_operation($mbean_props),
                        args => $self->complete->mbean_operations($mbean_props),
                       },
-            "pwd" => { 
-                      desc => "Show the currently selected MBean",
-                      proc => sub {
-                          my ($d,$k,$v,$r) = $self->color("domain_name","property_key","property_value","reset");
-                          print $d . $mbean_props->{domain} . $r . ":" . $self->_color_props($mbean_props) . "\n";
-                      }
-                     }             
+             
            };
 }
 
@@ -186,7 +163,7 @@ sub cmd_show_attributes {
                     $p .= sprintf(" $c_a%-31.31s$c_r\n",$attr);
                     $p .= $self->_dump($value);
                 } else {
-                    $p .= sprintf(" $c_a%-31.31s$c_r %s\n",$attr,$self->_dump_scalar($value));
+                    $p .= sprintf(" $c_a%-31.31s$c_r %s\n",$attr,$value);
                 }
             }
         } else {
@@ -195,7 +172,7 @@ sub cmd_show_attributes {
             if ($self->_is_object($value)) {
                 $p .= $self->_dump($value);
             } else {
-                $p .= $self->_dump_scalar($value)."\n";
+                $p .= $value."\n";
             }
         }
         $self->print_paged($p);
@@ -224,7 +201,7 @@ sub cmd_set_attribute {
             $p .= sprintf(" $c_l%-5.5ss$c_r\n","Old:");
             $p .= $self->_dump($old_value);
         } else {
-            $p .= sprintf(" $c_l%-5.5s$c_r %s\n","Old:",$self->_dump_scalar($old_value));
+            $p .= sprintf(" $c_l%-5.5s$c_r %s\n","Old:",$old_value);
         }
         $p .= sprintf(" $c_l%-5.5s$c_r %s\n","New:",$args[1]);;
         $self->print_paged($p);
@@ -252,7 +229,7 @@ sub cmd_execute_operation {
             $p .= sprintf(" $c_l%-7.7s$c_r\n","Return:");
             $p .= $self->_dump($value);
         } else {
-            $p .= sprintf(" $c_l%-7.7s$c_r %s\n","Return:",$self->_dump_scalar($value));
+            $p .= sprintf(" $c_l%-7.7s$c_r %s\n","Return:",$value);
         }
         $self->print_paged($p);
     }
@@ -261,38 +238,22 @@ sub cmd_execute_operation {
 sub _is_object {
     my $self = shift;
     my $value = shift;
-    return JMX::Jmx4Perl::Util->is_object_to_dump($value);
+    return ref($value) && !JSON::is_bool($value);
 }
 
 sub _dump {
     my $self = shift;
     my $value = shift;
-    return JMX::Jmx4Perl::Util->dump_value($value,{format => $self->_get_opt_or_config("format"),
-                                                   booleans => $self->_get_opt_or_config("booleans"),
-                                                   indent => $self->_get_opt_or_config("indent")});
+    local $Data::Dumper::Terse = 1;
+    local $Data::Dumper::Indent = 1;
+#   local $Data::Dumper::Useqq = 1;
+    local $Data::Dumper::Deparse = 1;
+    local $Data::Dumper::Quotekeys = 0;
+    local $Data::Dumper::Sortkeys = 1;
+    my $ret = Dumper($value);
+    $ret =~ s/^/   /gm;
+    return $ret;
 }
-
-sub _dump_scalar {
-    my $self = shift;
-    return JMX::Jmx4Perl::Util->dump_scalar(shift,$self->_get_opt_or_config("booleans"));
-}
-
-
-sub _get_opt_or_config {
-    my $self = shift;
-    my $key = shift;
-
-    my $args = $self->context->args || {};
-    my $config = $self->context->config || {};
-    if (defined($args->{option}) && defined($args->{option}->{lc $key})) {
-        return $args->{option}->{lc $key};
-    } else {
-        my $shell_config = $config->{shell} || {};
-        return $shell_config->{lc $key};
-    }
-}
-
-
 # =================================================================================================== 
 
 =item cmd_list
@@ -318,7 +279,7 @@ sub cmd_list_domains {
     return sub {
         my $context = $self->context;
         my $agent = $context->agent;
-        print "Not connected to a server\n" and return unless $agent;      
+        print "Not connected to a server\n" and return unless $agent;        
         my ($opts,@filters) = $self->extract_command_options(["l!"],@_);
         if ($domain) {
             if (@filters) {
@@ -349,8 +310,7 @@ sub cmd_show_mbean {
         my $info = $m_info->{info};
         my ($c_m,$c_a,$c_o,$c_r) = $self->color("mbean_name","attribute_name","operation_name","reset");
         my $op_len = 50 + length($c_o) + length($c_r);
-        my ($do_show_attrs,$do_show_ops,$filters) = $self->_show_what_from_mbean($info,@_);
-
+        
         my $p = "";
         
         my $name = $m_info->{full};
@@ -358,88 +318,46 @@ sub cmd_show_mbean {
         $p .= "\n\n";
         
         #print Dumper($m_info);
+
         my $attrs = $info->{attr};
-        if ($do_show_attrs) {
-            my @lines = ();
+        if ($attrs && keys %$attrs) {
+            $p .= "Attributes:\n";
             for my $attr (keys %$attrs) {
-                my $line = "";                
-                if ($self->_pass_filter($attr,$filters)) {
-                    if (length($attr) > 31) {
-                        $line .= sprintf("  $c_a%s$c_r\n",$attr);
-                        $line .= sprintf("  %-31.31s %-13.13s %-4.4s %s\n",
-                                      $self->_pretty_print_type($attrs->{$attr}->{type}),
-                                      $attrs->{$attr}->{rw} eq "false" ? "[ro]" : "",$attrs->{$attr}->{desc});
-                    } else {
-                        $line .= sprintf("  $c_a%-31.31s$c_r %-13.13s %-4.4s %s\n",$attr,
-                                      $self->_pretty_print_type($attrs->{$attr}->{type}),
-                                      $attrs->{$attr}->{rw} eq "false" ? "[ro]" : "",$attrs->{$attr}->{desc});
-                    }
-                    push @lines,$line;
+                if (length($attr) > 31) {
+                    $p .= sprintf("  $c_a%s$c_r\n",$attr);
+                    $p .= sprintf("  %-31.31s %-13.13s %-4.4s %s\n",
+                                  $self->_pretty_print_type($attrs->{$attr}->{type}),
+                                  $attrs->{$attr}->{rw} eq "false" ? "[ro]" : "",$attrs->{$attr}->{desc});
+                } else {
+                    $p .= sprintf("  $c_a%-31.31s$c_r %-13.13s %-4.4s %s\n",$attr,
+                                  $self->_pretty_print_type($attrs->{$attr}->{type}),
+                                  $attrs->{$attr}->{rw} eq "false" ? "[ro]" : "",$attrs->{$attr}->{desc});
                 }
             }
-            if (@lines) {
-                $p .= "Attributes:\n";
-                $p .= join "",@lines;
-                $p .= "\n";
-            }
+            $p .= "\n";
         }
         my $ops = $info->{op};
-        if ($do_show_ops) {
-            my @lines = ();
+        if ($ops && keys %$ops) {
+            $p .= "Operations:\n";
             for my $op (keys %$ops) {
-                my $line = "";
-                if ($self->_pass_filter($op,$filters)) {
-                    my $overloaded = ref($ops->{$op}) eq "ARRAY" ? $ops->{$op} : [ $ops->{$op} ];
-                    for my $m_info (@$overloaded) {
-                        my $sig = $self->_signature_to_print($op,$m_info);
-                        if (length($sig) > $op_len) {
-                            $line .= sprintf("  %s\n",$sig);
-                            $line .= sprintf("  %-50.50s %s\n","",$m_info->{desc}) if $m_info->{desc};                        
-                        } else {
-                            $line .= sprintf("  %-${op_len}.${op_len}s %s\n",$sig,$m_info->{desc});
-                        }
+                my $overloaded = ref($ops->{$op}) eq "ARRAY" ? $ops->{$op} : [ $ops->{$op} ];
+                for my $m_info (@$overloaded) {
+                    my $sig = $self->_signature_to_print($op,$m_info);
+                    if (length($sig) > $op_len) {
+                        $p .= sprintf("  %s\n",$sig);
+                        $p .= sprintf("  %-50.50s %s\n","",$m_info->{desc}) if $m_info->{desc};                        
+                    } else {
+                        $p .= sprintf("  %-${op_len}.${op_len}s %s\n",$sig,$m_info->{desc});
                     }
-                    push @lines,$line;
                 }
             }
-            if (@lines) {
-                $p .= "Operations:\n";
-                $p .= join "",@lines;
-                $p .= "\n";            
-            }
+            $p .= "\n";
         }
         $self->print_paged($p);
         #print Dumper($info);
     }
 }
 
-sub _pass_filter {
-    my $self = shift;
-    my $check = shift;
-    my $regexps = shift;
-    return 1 unless @$regexps;
-    for my $regexp (@$regexps) {
-        return 1 if $check =~ $regexp;
-    }
-    return 0;
-}
-
-sub _show_what_from_mbean {
-    my $self = shift;
-    my ($info,@args) = @_;
-    my ($opts,@filter) = $self->extract_command_options(["attributes|a!","operations|ops|o!"],@args);
-    my $no_restrict = !defined($opts->{attributes}) && !defined($opts->{operations}); 
-    my $show_attrs = $info->{attr} && keys %{$info->{attr}} && ($opts->{attributes} || $no_restrict);
-    my $show_ops = $info->{op} && keys %{$info->{op}} && ($opts->{operations} || $no_restrict);
-    my @filter_regexp = map {
-        s/\*/.*/g;  
-        s/\?/./g; 
-        my $f = '^' . $_ . '$';
-        qr/$f/i
-    } @filter;
-    return ($show_attrs,$show_ops,\@filter_regexp);
-}
-
 sub _line_aligned {
     my $self = shift;
     my $max_lengths = shift;
@@ -591,28 +509,6 @@ sub _filter {
 
 =cut
 
-sub _cd_absolute {
-    my $self = shift;
-    my $domain = shift;
-    my $props;
-
-    if ($domain) {
-        $domain =~ s/:+$//;
-        ($domain,$props) = split(/:/,$domain,2) if $domain =~ /:/;
-    }    
-    die "No domain $domain\n" unless $self->_check_domain($domain);
-    $self->_check_mbean($domain,$props) if $props;
-    $self->reset_stack;
-    $self->_cd_domain($domain);
-    $self->_cd_mbean($domain,$props) if $props;
-}
-
-sub _check_mbean {
-    my $self = shift;
-    my $domain = shift;
-    my $props = shift;
-    $self->_get_mbean($domain,$props) || die "No MBean $domain:$props\n";
-}
 
 sub _cd_domain {
     my $self = shift;
@@ -626,8 +522,9 @@ sub _cd_mbean {
     my $self = shift;
     my $domain = shift;
     my $mbean = shift;
-    
-    my $mbean_props = $self->_check_mbean($domain,$mbean);
+
+    my $mbean_props = $self->_get_mbean($domain,$mbean);
+    die "No MBean $domain:$mbean\n" unless $mbean_props; 
     my $mbean_cmds = $self->mbean_commands($mbean_props);
     &{$self->push_on_stack($mbean_props->{prompt},$mbean_cmds)};    
 }
@@ -643,23 +540,9 @@ sub _check_domain {
 sub _get_mbean {
     my $self = shift;
     my $domain = shift;
-    my $props = shift;
+    my $mbean = shift;
     my $context = $self->context;
-    if ($props =~ /\*/) {
-        my $mbeans = $context->search_mbeans($domain . ":" . $props);
-        # TODO: If more than one, present a menu to select from. Now simply die
-        return undef unless @{$mbeans};
-        if (scalar(@$mbeans) > 1) {
-            my $toomany = "";
-            for my $m (@$mbeans) {
-                my ($s,$r) = $self->color("mbean_name","reset");                          
-                $toomany .=" >>> " . $s . $m->{full} . $r . "\n";
-            }
-            die "More than one MBean found:\n" . $toomany;
-        } 
-        return $mbeans->[0];
-    }
-    return $context->mbeans_by_name->{$domain . ":" . $props};
+    return $context->mbeans_by_name->{$domain . ":" . $mbean};
 }
 
 # Handle navigational commands
