@@ -59,20 +59,14 @@ are additonal arguments used for exec-operations,
 Multiple request object are returned e.g. if a relative check has to be
 performed in order to get the base value as well.
 
-The returned array can contain coderefs which should be executed directly and
-its return value should be used in order to perfoorm the check. 
-
 =cut
 
 sub get_requests {
     my $self = shift;
     my $jmx = shift;
     my $args = shift;
-    # If a script is given, extract a subref and return it
-    return [ $self->_extract_script_as_subref($jmx) ] if $self->script;
 
     my $do_read = $self->attribute || $self->value;
-    my $do_exec = $self->operation;
     if ($self->alias) {
         my $alias = JMX::Jmx4Perl::Alias->by_name($self->alias);
         die "No alias '",$self->alias," known" unless $alias;
@@ -82,10 +76,8 @@ sub get_requests {
     my $request;
     if ($do_read) {
         $request = JMX::Jmx4Perl::Request->new(READ,$self->_prepare_read_args($jmx));
-    } elsif ($do_exec) {
-        $request = JMX::Jmx4Perl::Request->new(EXEC,$self->_prepare_exec_args($jmx,@$args));
     } else {
-        die "Neither an attribute/value, an operation or a script given";
+        $request = JMX::Jmx4Perl::Request->new(EXEC,$self->_prepare_exec_args($jmx,@$args));
     }
     my $method = $self->{np}->opts->{method} || $self->{config}->{method};
     if ($method) {
@@ -93,20 +85,14 @@ sub get_requests {
     }
     push @requests,$request;
 
-    if ($self->base || $self->base_mbean) {
+    if ($self->base) {
         if (!looks_like_number($self->base)) {
             # It looks like a number, so we will use the base literally
-            my $alias;
-            
-            if ($self->base) {
-                $alias = JMX::Jmx4Perl::Alias->by_name($self->base);
-            }
+            my $alias = JMX::Jmx4Perl::Alias->by_name($self->base);
             if ($alias) {
                 push @requests,new JMX::Jmx4Perl::Request(READ,$jmx->resolve_alias($self->base));
             } else {
-                my ($mbean,$attr,$path) = $self->base_mbean ? 
-                  ($self->base_mbean, $self->base_attribute, $self->base_path) : 
-                    $self->_split_attr_spec($self->base);
+                my ($mbean,$attr,$path) = $self->_split_attr_spec($self->base);
                 die "No MBean given in base name ",$self->base unless $mbean;
                 die "No Attribute given in base name ",$self->base unless $attr;
                 
@@ -121,25 +107,6 @@ sub get_requests {
     return \@requests;
 }
 
-# Create a subref where all params from the outside are available as closures.
-sub _extract_script_as_subref {
-    my $self = shift;
-    my $jmx = shift;
-    my $script = $self->script || die "No script given";
-    my $full_script = <<"EOT";
-sub {
-  my \$j4p = shift;
-  return sub { 
-     $script
-  }
-}
-EOT
-    #print $full_script,"\n";
-    my $sub = eval $full_script;
-    die "Cannot eval script for check ",$self->name,": $@" if $@;
-    return &$sub($jmx);
-}
-
 =item $single_check->exract_responses($responses,$requests,$target)
 
 Extract L<JMX::Jmx4Perl::Response> objects and add the deducted results to 
@@ -162,27 +129,18 @@ sub extract_responses {
     my $requests = shift;
     my $opts = shift || {};
     my $np = $self->{np};
-    my $msg_handler = $np->{msg_handler} || $np; 
 
     # Get response/request pair
     my $resp = shift @{$responses};
     my $request = shift @{$requests};
     #print Dumper($resp);
     my @extra_requests = ();
-    my $value;
-    my $script_mode = undef;
-    if (ref($request) eq "CODE") {
-        # It's a script, so the 'response' is already the value
-        $script_mode = 1;
-        $value = $resp;
-    } else {
-        $self->_verify_response($request,$resp);
-        $value = $self->_extract_value($request,$resp);
-    }
+    $self->_verify_response($request,$resp);
+    my $value = $self->_extract_value($request,$resp);
    
     # Delta handling
     my $delta = $self->delta;
-    if (defined($delta) && !$script_mode) {
+    if (defined($delta)) {
         $value = $self->_delta_value($request,$resp,$delta);
         unless (defined($value)) {
             push @extra_requests,$self->_switch_on_history($request,$opts->{target});
@@ -193,66 +151,46 @@ sub extract_responses {
     # Normalize value 
     my ($value_conv,$unit) = $self->_normalize_value($value);
     my $label = $self->_get_name(cleanup => 1);
-    if ( ($self->base || $self->base_mbean) && !$script_mode) {
+    if ($self->base) {
         # Calc relative value 
         my $base_value = $self->_base_value($self->base,$responses,$requests);
         my $rel_value = sprintf "%2.2f",$base_value ? (int((($value / $base_value) * 10000) + 0.5) / 100) : 0;
                 
         # Performance data. Convert to absolute values before
-        if ($self->_include_perf_data) {
-            if ($self->perfdata && $self->perfdata =~ /^\s*\%\s*/) {
-                $np->add_perfdata(label => $label, value => $rel_value, uom => '%',
-                                  critical => $self->critical, warning => $self->warning);
-            } else {
-                my ($critical,$warning) = $self->_convert_relative_to_absolute($base_value,$self->critical,$self->warning);
-                $np->add_perfdata(label => $label,value => $value,
-                                  critical => $critical,warning => $warning,
-                                  min => 0,max => $base_value,
-                                  $self->unit ? (uom => $self->unit) : ());
-            }
-        }
+        my ($critical,$warning) = $self->_convert_relative_to_absolute($base_value,$self->critical,$self->warning);
+        $np->add_perfdata(label => $label,value => $value,
+                          critical => $critical,warning => $warning,
+                          min => 0,max => $base_value,
+                          $self->unit ? (uom => $self->unit) : ());
         # Do the real check.
-        my ($code,$mode) = $self->_check_threshold($rel_value);
+        my ($code,$mode) = $self->_check_threshhold($rel_value);
         # For Multichecks, we remember the label of a currently failed check
-        $self->update_error_stats($opts->{error_stat},$code) unless $code == OK;
+        $self->_update_error_stats($opts->{error_stat},$code) unless $code == OK;
         my ($base_conv,$base_unit) = $self->_normalize_value($base_value);
-        $msg_handler->add_message($code,$self->_exit_message(code => $code,mode => $mode,rel_value => $rel_value, 
-                                                    value => $value_conv, unit => $unit, base => $base_conv, 
+        $np->add_message($code,$self->_exit_message(code => $code,mode => $mode,rel_value => $rel_value, 
+                                                    value => $value_conv, unit => $unit,base => $base_conv, 
                                                     base_unit => $base_unit, prefix => $opts->{prefix}));            
     } else {
         # Performance data
-        $value = $self->_sanitize_value($value);
-        if ($self->_include_perf_data) {
-            $np->add_perfdata(label => $label,
-                              critical => $self->critical, warning => $self->warning, 
-                              value => $value,$self->unit ? (uom => $self->unit) : ());
-        }
+        $np->add_perfdata(label => $label,
+                          critical => $self->critical, warning => $self->warning, 
+                          value => $value,$self->unit ? (uom => $self->unit) : ());
         
         # Do the real check.
-        my ($code,$mode) = $self->_check_threshold($value);
-        $self->update_error_stats($opts->{error_stat},$code) unless $code == OK;
-        $msg_handler->add_message($code,$self->_exit_message(code => $code,mode => $mode,value => $value_conv, unit => $unit,
-                                                             prefix => $opts->{prefix}));                    
+        my ($code,$mode) = $self->_check_threshhold($value);
+        $self->_update_error_stats($opts->{error_stat},$code) unless $code == OK;
+        $np->add_message($code,$self->_exit_message(code => $code,mode => $mode,value => $value_conv, unit => $unit,
+                                                    prefix => $opts->{prefix}));                    
     }
     return @extra_requests;
 }
 
-sub _include_perf_data {
-    my $self = shift;
-    # No perf dara for string based checks by default
-    my $default = not defined($self->string);
-    # If 'PerfData' is set explicitely to false/off/no/0 then no perfdata
-    # will be included
-    return $default unless defined($self->perfdata);
-    return $self->perfdata !~ /^\s*(false|off|no|0)\s*$/i;
-}
-
-sub update_error_stats {
+sub _update_error_stats {
     my $self = shift;
     my $error_stat = shift || return;
     my $code = shift;
 
-    my $label = $self->{config}->{name} || $self->{config}->{key};
+    my $label = $self->{config}->{key} || $self->{config}->{name};
     if ($label) {
         my $arr = $error_stat->{$code} || [];
         push @$arr,$label;
@@ -299,12 +237,12 @@ sub _extract_value_from_pattern_request {
     my $self = shift;
     my $val = shift;
     my $np = $self->{np};
-    $self->_die("Pattern request does not result in a proper return format: " . Dumper($val))
+    $self->nagios_die("Pattern request does not result in a proper return format: " . Dumper($val))
       if (ref($val) ne "HASH");
-    $self->_die("More than one MBean found for a pattern request: " . Dumper([keys %$val])) if keys %$val != 1;
+    $self->nagios_die("More than one MBean found for a pattern request: " . Dumper([keys %$val])) if keys %$val != 1;
     my $attr_val = (values(%$val))[0];
-    $self->_die("Invalid response for pattern match: " . Dumper($attr_val)) unless ref($attr_val) eq "HASH";
-    $self->_die("Only a single attribute can be used. Given: " . Dumper([keys %$attr_val])) if keys %$attr_val != 1;
+    $self->nagios_die("Invalid response for pattern match: " . Dumper($attr_val)) unless ref($attr_val) eq "HASH";
+    $self->nagios_die("Only a single attribute can be used. Given: " . Dumper([keys %$attr_val])) if keys %$attr_val != 1;
     return $self->_null_safe_value((values(%$attr_val))[0]);
 }
 
@@ -324,7 +262,7 @@ sub _delta_value {
         }
         if (!@$hist_val) {
             # Can happen in some scenarios when requesting the first history entry,
-            # we return 0 here
+            # we are return 0 here
             return 0;
         }
         my $old_value = $hist_val->[0]->{value};
@@ -359,13 +297,12 @@ sub _base_value {
     my $requests = shift;
 
     if (looks_like_number($name)) {
-        # It looks like a number, so we suppose its the base value itself
+        # It looks like a number, so we suppose its  the base value itself
         return $name;
     }
     my $resp = shift @{$responses};
     my $req = shift @{$requests};
-    $self->_die($resp->{error}) if $resp->{error};
-    #print Dumper($req,$resp);
+    $self->nagios_die($resp->{error}) if $resp->{error};
     return $self->_extract_value($req,$resp);
 }
 
@@ -430,28 +367,18 @@ sub _normalize_value {
     die "Unknown unit '$unit' for value $value";
 }
 
-sub _sanitize_value {
-    my ($self,$value) = @_;
-    if ($value =~ /\de/i) {
-        $value = sprintf("%f", $value);
-    }
-    return $value;
-}
-
 sub _verify_response {
     my ($self,$req,$resp) = @_;
     my $np = $self->{np};
     if ($resp->is_error) {
+        my $stacktrace = $resp->stacktrace;
         my $extra = "";
-        if ($np->opts->{verbose}) {
-            my $stacktrace = $resp->stacktrace;
-            $extra = ref($stacktrace) eq "ARRAY" ? join "\n",@$stacktrace : $stacktrace if $stacktrace;
-        }
-        $self->_die("Error: ".$resp->status." ".$resp->error_text.$extra);
+        $extra = ref($stacktrace) eq "ARRAY" ? join "\n",@$stacktrace : $stacktrace if $stacktrace;
+        $self->nagios_die("Error: ".$resp->status." ".$resp->error_text.$extra);
     }
     
     if (!$req->is_mbean_pattern && (ref($resp->value) && !$self->string) && !JSON::is_bool($resp->value)) { 
-        $self->_die("Response value is a " . ref($resp->value) .
+        $self->nagios_die("Response value is a " . ref($resp->value) .
                         ", not a plain value. Did you forget a --path parameter ?". " Value: " . 
                         Dumper($resp->value));
     }
@@ -471,7 +398,7 @@ sub _get_name {
             } else {
                 my $val = $self->value;
                 if ($val) {
-                    $name = "[" . $val . "]";
+                    $name = "[" . $self->value . "]";
                 } else {
                     my $a_or_o = $self->attribute || $self->operation || "";
                     my $p = $self->path ? "," . $self->path : "";
@@ -498,7 +425,7 @@ sub _prepare_read_args {
 
     if ($self->alias) {
         my @req_args = $jmx->resolve_alias($self->alias);
-        $self->_die("Cannot resolve attribute alias ",$self->alias()) unless @req_args > 0;
+        $self->nagios_die("Cannot resolve attribute alias ",$self->alias()) unless @req_args > 0;
         if ($self->path) {
             @req_args == 2 ? $req_args[2] = $self->path : $req_args[2] .= "/" . $self->path;
         }
@@ -518,9 +445,7 @@ sub _prepare_exec_args {
     # Merge CLI arguments and arguments from the configuration,
     # with CLI arguments taking precedence
     my @cli_args = @_;
-    my $config_args = $self->{config}->{argument};
-    
-    $config_args = [ $config_args ] if defined($config_args) && !ref($config_args) eq "ARRAY";
+    my $config_args = $self->{config}->{args};
     my @args = ();
     if ($config_args) {
         my @c_args = (@$config_args);
@@ -534,7 +459,7 @@ sub _prepare_exec_args {
     }
     if ($self->alias) {
         my @req_args = $jmx->resolve_alias($self->alias);
-        $self->_die("Cannot resolve operation alias ",$self->alias()) unless @req_args >= 2;
+        $self->nagios_die("Cannot resolve operation alias ",$self->alias()) unless @req_args >= 2;
         return (@req_args,@args);
     } else {
         return ($self->mbean,$self->operation,@args);
@@ -554,10 +479,10 @@ sub _split_attr_spec {
         $p =~ s|\\(.)|$1|sg;
         push @ret,$p;
     }    
-    return (shift(@ret),shift(@ret),@ret ? join("/",@ret) : undef);
+    return @ret;
 }
 
-sub _check_threshold {
+sub _check_threshhold {
     my $self = shift;
     my $value = shift;
     my $np = $self->{np};
@@ -571,11 +496,10 @@ sub _check_threshold {
         # Verify numeric thresholds
         my @ths = 
           (
-           defined($self->critical) ? (critical => $self->critical) : (),
-           defined($self->warning) ? (warning => $self->warning) : ()
-          );  
-        #print Dumper({check => $value,@ths});
-        return (@ths ? $np->check_threshold(check => $value,@ths) : OK,"numeric");    
+           $self->critical ? (critical => $self->critical) : (),
+           $self->warning ? (warning => $self->warning) : ()
+          );            
+        return ($np->check_threshold(check => $value,@ths),"numeric");    
     } else {
         return
           ($self->_check_string_threshold($value,CRITICAL,$self->critical) ||
@@ -621,7 +545,7 @@ sub _exit_message {
     my $code = $args->{code};
     my $mode = $args->{mode};
     if ($code == CRITICAL || $code == WARNING) {
-        if ($self->base || $self->base_mbean) {
+        if ($self->base) {
             return $self->_format_label
               ('%n : Threshold \'%t\' failed for value %.2r% ('. &_placeholder($args,"v") .' %u / '.
                &_placeholder($args,"b") . ' %u)',$args);
@@ -634,7 +558,7 @@ sub _exit_message {
             }
         }
     } else {
-        if ($self->base || $self->base_mbean) {
+        if ($self->base) {
             return $self->_format_label('%n : In range %.2r% ('. &_placeholder($args,"v") .' %u / '.
                                         &_placeholder($args,"b") . ' %w)',$args);
         } else {
@@ -663,26 +587,20 @@ sub _format_label {
     my $self = shift;
     my $label = shift;
     my $args = shift;
-    # %r : relative value (as percent)
-    # %q : relative value (as floating point)
+    # %r : relative value
     # %v : value
-    # %f : value as floating point
     # %u : unit
     # %b : base value
-    # %w : base unit
     # %t : threshold failed ("" for OK or UNKNOWN)
     # %c : code ("OK", "WARNING", "CRITICAL", "UNKNOWN")
     # %d : delta
-    # 
     my @parts = split /(\%[\w\.\-]*\w)/,$label;
     my $ret = "";
     foreach my $p (@parts) {
         if ($p =~ /^(\%[\w\.\-]*)(\w)$/) {
             my ($format,$what) = ($1,$2);
-            if ($what eq "r" || $what eq "q") {
-                my $val = $args->{rel_value} || 0;
-                $val = $what eq "r" ? $val : $val / 100; 
-                $ret .= sprintf $format . "f",$val;
+            if ($what eq "r") {
+                $ret .= sprintf $format . "f",($args->{rel_value} || 0);
             } elsif ($what eq "b") {
                 $ret .= sprintf $format . &_format_char($args->{base}),($args->{base} || 0);
             } elsif ($what eq "u" || $what eq "w") {
@@ -691,7 +609,11 @@ sub _format_label {
             } elsif ($what eq "f") {
                 $ret .= sprintf $format . "f",$args->{value};
             } elsif ($what eq "v") {
-                $ret .= &_format_value($format,$args->{mode},$args->{value});
+                if ($args->{mode} ne "numeric") {
+                    $ret .= sprintf $format . "s",$args->{value};
+                } else {
+                    $ret .= sprintf $format . &_format_char($args->{value}),$args->{value};
+                }
             } elsif ($what eq "t") {
                 my $code = $args->{code};
                 my $val = $code == CRITICAL ? $self->critical : ($code == WARNING ? $self->warning : "");
@@ -702,10 +624,6 @@ sub _format_label {
                 $ret .= sprintf $format . "s",$self->_get_name();
             } elsif ($what eq "d") {
                 $ret .= sprintf $format . "d",$self->delta;
-            } elsif ($what eq "y") {
-                $ret .= &_format_value($format,$args->{mode},$self->warning);
-            } elsif ($what eq "z") {
-                $ret .= &_format_value($format,$args->{mode},$self->critical);                
             }
         } else {
             $ret .= $p;
@@ -720,25 +638,16 @@ sub _format_label {
     }
 }
 
-sub _format_value {
-    my $format = shift;
-    my $mode = shift;
-    my $value = shift;
-    if ($mode ne "numeric") {
-        return sprintf $format . "s",$value;
-    } else {
-        return sprintf $format . &_format_char($value),$value;
-    }    
-}
 sub _format_char {
     my $val = shift;
     $val =~ /\./ ? "f" : "d";
 }
 
-sub _die {
+sub nagios_die {
     my $self = shift;
     my $msg = join("",@_);
-    die $msg,"\n";
+    my $np = $self->{np};
+    $np->nagios_die($msg,$np->opts->{'unknown-is-critical'} ? CRITICAL : UNKNOWN);
 }
 
 my $CHECK_CONFIG_KEYS = {
@@ -752,28 +661,23 @@ my $CHECK_CONFIG_KEYS = {
                          "delta" => "delta",
                          "name" => "name",
                          "base" => "base",
-                         "base-mbean" => "basembean",
-                         "base-attribute" => "baseattribute",
-                         "base-path" => "basepath",
                          "unit" => "unit",
                          "numeric" => "numeric",
                          "string" => "string",
                          "label" => "label",
-                         "perfdata" => "perfdata",
+                         # New:
                          "value" => "value",
                          "null" => "null",
-                         "script" => "script"
                         };
 
 
 # Get the proper configuration values
-
 sub AUTOLOAD {
     my $self = shift;
     my $np = $self->{np};
     my $name = $AUTOLOAD;
     $name =~ s/.*://;   # strip fully-qualified portion
-    $name =~ s/_/-/g;
+    $name =~ s/-/_/g;
 
     if ($CHECK_CONFIG_KEYS->{$name}) {
         return $np->opts->{$name} if defined($np->opts->{$name});
@@ -783,7 +687,7 @@ sub AUTOLOAD {
             return undef;
         }
     } else {
-        $self->_die("No config attribute \"" . $name . "\" known");
+        $self->nagios_die("No config attribute \"" . $name . "\" known");
     }
 }
 
